import re

import pkg_resources

from fabric.api import sudo, task
from fabric.contrib import files
from fabric.context_managers import settings

from . import apt


__all__ = 'distro_install', 'find_current_version', 'install_systemd'


APT_KEYS = [
    'EA312927',
]


@task
def distro_install(version="3.2"):
    """
    Install mongodb as an apt package (which also configures it as a
    service).

    TODO: Currently, the service doesn't start on Ubuntu 15.04 or later.
    See https://www.digitalocean.com/community/tutorials/how-to-install-mongodb-on-ubuntu-16-04
    for more details on how to configure it as a service.
    """

    if version.startswith('2.'):
        return distro_install_2(version)

    elif version.startswith('3.'):
        return distro_install_3(version)

    raise RuntimeError('Unknown version {}'.format(version))


def distro_install_2(version):
    """
    Install MongoDB version 2.x
    """
    assert version.startswith('2.')

    # MongoDB 2
    content = (
        'deb http://downloads-distro.mongodb.org/repo/ubuntu-upstart '
        'dist 10gen',
    )
    org_list = '/etc/apt/sources.list.d/mongodb.list'
    files.append(org_list, content, use_sudo=True)


@task
def find_current_version():
    output = sudo('apt list -qq mongodb-org')
    return re.search(r'\d+\.\d+\.\d+', output).group(0)


def distro_install_3(version):
    """
    Install MongoDB version 3.x
    """
    assert version.startswith('3.')

    lsb_release = apt.lsb_release()
    repo_url = "http://repo.mongodb.org/apt/ubuntu"
    tmpl = "deb {repo_url} {lsb_release}/mongodb-org/{version} multiverse"
    content = tmpl.format(**locals())
    org_list = '/etc/apt/sources.list.d/mongodb-org-{version}.list'.format(**locals())
    files.append(org_list, content, use_sudo=True)

    with settings(warn_only=True):
        for key in APT_KEYS:
            sudo('apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv {}'.format(key))

    sudo('apt update')
    version = find_current_version()
    apt.install_packages('mongodb-org={version}'.format(**locals()))
    install_systemd()


@task
def install_systemd():
    """
    On newer versions of Ubuntu, make sure that systemd is configured
    to manage the service.
    https://docs.mongodb.com/v3.2/tutorial/install-mongodb-on-ubuntu/#ubuntu-16-04-only-create-systemd-service-file
    """
    if apt.lsb_version() < '15.04':
        return

    fn = 'mongod.service'
    service_strm = pkg_resources.resource_stream(__name__, fn)
    files.put(service_strm, '/lib/systemd/system/' + fn, use_sudo=True)
    # TODO: does the service start automatically? If not,
    # sudo('systemctl start mongod')



"""
This package has no tests, but for the automatic deployment
to occur, tests need to run and pass. If you can think of a
useful test to add to the package, please do so, and then this
module can be removed.
"""

def test_null():
	"A trivial test to enable the test runner to pass"



from __future__ import print_function

import io
import re
import contextlib

import jaraco.apt
from jaraco.itertools import always_iterable
from fabric.operations import sudo, run
from fabric.api import task, put
from fabric.context_managers import settings
import fabric.utils


__all__ = [
    'install_packages',
    'create_installers_group',
    'add_installer',
    'add_ppa',
]


@contextlib.contextmanager
def package_context(target, action='install'):
    """
    A context for installing the build dependencies for a given target (or
    targets). Uses apt. Removes the dependencies when the context is
    exited. One may prevent the removal of some or all packages by modifying
    the list within the context.
    """
    target = ' '.join(always_iterable(target))
    status = sudo('apt {action} -q -y {target}'.format(**vars()))
    packages = jaraco.apt.parse_new_packages(status)
    try:
        yield packages
    finally:
        remove_packages(packages)

@task
def install_packages(*packages):
    with package_context(packages) as to_remove:
        installed = list(to_remove)
        to_remove[:] = []
    return installed

def build_dependency_context(target):
    return package_context(target, 'build-dep')

def remove_packages(packages):
    if not packages:
        print("No packages specified, nothing to remove")
        return
    sudo('apt autoremove -y -q ' + ' '.join(packages))

@task
def create_installers_group():
    """
    Create an 'installers' group that has rights to install/remove software
    without typing a password.
    """
    apt_commands = ['aptitude', 'apt-get', 'dpkg', 'apt-key',
            'apt-add-repository', 'apt-cache', 'apt']
    commands = ', '.join('/usr/bin/' + cmd for cmd in apt_commands)
    content = "%installers ALL=NOPASSWD: {commands}\n".format(**locals())
    upload_sudoersd_file('installers', content)
    with settings(warn_only=True):
        sudo('addgroup installers')
    print("Grant installation privilege with 'usermod -a -G installers $username' or yg-fab add_installer:$username")

def upload_sudoersd_file(name, content):
    """
    Thanks to a long-standing bug in Ubuntu Lucid
    (https://bugs.launchpad.net/ubuntu/+source/sudo/+bug/553786),
    we have to take special precaution when creating sudoers.d files.
    """
    stream = io.BytesIO(content.encode('utf-8'))
    tmp_name = '/tmp/'+ name
    put(stream, tmp_name, mode=0o440)
    sudo('chown root:root ' + tmp_name)
    sudo('mv {tmp_name} /etc/sudoers.d'.format(**vars()))

@task
def add_installer(username):
    """
    Add username to the installers group, after which they should be able to
    install/remove software without typing a password.
    """
    sudo("usermod -a -G installers {username}".format(**vars()))

def ubuntu_version():
    pattern = re.compile('Ubuntu ([\d.]+)')
    out = run('cat /etc/issue')
    return pattern.match(out).group(1)

@task
def add_ppa(name):
    """
    Add the Personal Package Archive
    """
    sudo('apt update')
    # need python-software-properties for apt-add-repository
    sudo('apt install -q -y python-software-properties')
    # apt-add-repository returns 0 even when it failed, so check its output
    #  for success or failure.
    cmd = [
        'apt-add-repository',
        'ppa:' + name,
    ]
    if ubuntu_version() >= '12.':
        cmd[1:1] = ['-y']
    res = sudo(' '.join(cmd))
    if not 'Total number processed: 1' in res:
        msg = "Failed to add PPA {name}".format(**vars())
        fabric.utils.abort(msg)
    sudo('apt update')

def lsb_release():
    return run("lsb_release -sc").strip()


def lsb_version():
    return run('lsb_release -sr').strip()
